/*---------------------------------------------------------------------------*/
/* GIVE-RMS.C -- contains RMS routines for GIVE.C                            */
/*---------------------------------------------------------------------------*/

#include <rms.h>
#include <descrip.h>
#include <ssdef.h>
#include <string.h>
#include "give.h"
#include "give-rms.h"

#define K_RMS_BUFFERSIZE 2048

/*---------------------------------------------------------------------------*/
/* Get_File_Size -- determines the size, in blocks of a file                 */

ulong Get_File_Size( char *filename, ulong *size )
{
   ulong ss_stat;

   struct FAB fab;
   struct NAM nam;

   fab = cc$rms_fab;
   nam = cc$rms_nam;

   fab.fab$l_fna = filename;
   fab.fab$b_fns = strlen( filename );
   fab.fab$l_nam = &nam;

   ss_stat = sys$open( &fab, 0, 0 );
   if( ss_failure( ss_stat ) )
      return( ss_stat );

   *size = fab.fab$l_alq;
   
   ss_stat = sys$close( &fab, 0, 0 );
   if( ss_failure( ss_stat ) )
      return( ss_stat );

   return( SS$_NORMAL );
}

/*---------------------------------------------------------------------------*/
/* Extract_Filename -- this function parses filename into its root and its   */
/* extension.                                                                */

ulong Extract_Filename( char *filename, char *root, char *extension )
{
   ulong ss_stat;
   ulong i,j;

   char esa[NAM$C_MAXRSS+1];
   char rsa[NAM$C_MAXRSS+1];
   struct FAB fab;
   struct NAM nam;

   fab = cc$rms_fab;
   nam = cc$rms_nam;

   fab.fab$l_fna = filename;
   fab.fab$b_fns = strlen( filename );
   fab.fab$l_nam = &nam;
   nam.nam$l_esa = esa;
   nam.nam$b_ess = NAM$C_MAXRSS;
   nam.nam$l_rsa = rsa;
   nam.nam$b_rss = NAM$C_MAXRSS;

   ss_stat = sys$parse( &fab, 0, 0 );
   if( ss_failure( ss_stat ) )
      return( ss_stat );

   ss_stat = sys$search( &fab, 0, 0 );
   if( ss_failure( ss_stat ) )
      return( ss_stat );

   for( i=0; nam.nam$l_name[i] != '.'; i++ )
      root[i] = nam.nam$l_name[i];
   root[i++] = '\0';

   for( j=0; nam.nam$l_name[i+j] != ';'; j++ )
      extension[j] = nam.nam$l_name[i+j];
   extension[j] = '\0';

   while( ss_success( sys$search( &fab, 0, 0 )));

   return( SS$_NORMAL );   

}

/*---------------------------------------------------------------------------*/
/* Copy_File -- this function copies a file using RMS block IO calls         */

ulong Copy_File( char *source, char *destination )
{
   ulong ss_stat;
   char buffer[K_RMS_BUFFERSIZE];
   struct FAB s_fab, d_fab;
   struct RAB s_rab, d_rab;
   struct XABFHC s_fhc, d_fhc;

/*------*/
/* Initialize source file fields in fab, rab, and xab blocks.                */

   s_fab = cc$rms_fab;
   s_fab.fab$l_fna = source;
   s_fab.fab$b_fns = strlen( source );
   s_fab.fab$b_fac = FAB$M_GET | FAB$M_BIO;
   s_fab.fab$l_xab = &s_fhc;
   s_rab = cc$rms_rab;
   s_rab.rab$l_fab = &s_fab;
   s_rab.rab$l_ubf = buffer;
   s_rab.rab$w_usz = sizeof( buffer );
   s_fhc = cc$rms_xabfhc;

/*------*/
/* Open the source file and connect the record stream.                       */

   ss_stat = sys$open( &s_fab, 0, 0 );
   if( ss_failure( ss_stat ) )
      return( ss_stat );
   ss_stat = sys$connect( &s_rab, 0, 0 );
   if( ss_failure( ss_stat ) )
      return( ss_stat );

/*------*/
/* Setup the RMS blocks for the output file, copying appropriate fields from */
/* the input file.                                                           */
   
   d_fab = cc$rms_fab;
   d_fab.fab$b_fac = FAB$M_BIO | FAB$M_PUT;
   d_fab.fab$l_fna = destination;
   d_fab.fab$b_fns = strlen( destination );
   d_fab.fab$l_fop = s_fab.fab$l_fop;
   d_fab.fab$l_alq = s_fab.fab$l_alq;
   d_fab.fab$w_mrs = s_fab.fab$w_mrs;
   d_fab.fab$b_org = s_fab.fab$b_org;
   d_fab.fab$b_rat = s_fab.fab$b_rat;
   d_fab.fab$b_rfm = s_fab.fab$b_rfm;
   d_fab.fab$l_xab = &d_fhc;
   d_rab = cc$rms_rab;
   d_rab.rab$l_fab = &d_fab;
   d_rab.rab$b_rac = RAB$C_SEQ;
   d_rab.rab$l_rbf = buffer;
   d_fhc = cc$rms_xabfhc;
   d_fhc.xab$w_lrl = s_fhc.xab$w_lrl;

/*------*/
/* Now create the output file, and connect the record stream.                */

   ss_stat = sys$create( &d_fab, 0, 0 );
   if( ss_failure( ss_stat ) )
      return( ss_stat );
   ss_stat = sys$connect( &d_rab, 0, 0 );
   if( ss_failure( ss_stat ) )
      return( ss_stat );

/*------*/
/* Copy the file.                                                            */

   while( (ss_stat = sys$read( &s_rab, 0, 0 )) != RMS$_EOF) {
      d_rab.rab$w_rsz = s_rab.rab$w_rsz;
      ss_stat = sys$write( &d_rab, 0, 0 );
      if( ss_failure( ss_stat ) )
         return( ss_stat );
   }

/*------*/
/* Close up everything.                                                      */

   ss_stat = sys$close( &s_fab, 0, 0 );
   if( ss_failure( ss_stat ) )
      return( ss_stat );
   ss_stat = sys$close( &d_fab, 0, 0 );
   if( ss_failure( ss_stat ) )
      return( ss_stat );

   return( SS$_NORMAL );

}
